import {async, asyncMap} from '../index.es6';
import chai from 'chai';

var should = chai.should();

describe('function-async', function () {

    before(function (cb) {
        cb();
    });

    it('sequence flow', function (done) {

        async([
            function () {
                // sync result
                return 1;
            },
            function (result, next) {

                result.should.be.equal(1);

                // async result
                setTimeout(function () {
                    next(null, 2);
                }, 20);
            },
            function (result, next) {

                result.should.be.equal(2);

                // async result
                setTimeout(function () {
                    next(null, 3);
                }, 20);
            },
            function (result) {

                result.should.be.equal(3);

                return 4;
            }],
            done
        );
    });

    after(function () {
        console.log('DONE');
    });

});

describe('function-async errors', function () {

    before(function (cb) {
        cb();
    });
    it('throwing error', function (done) {

        async([
            function () {
                // go next with error
                throw new Error('error');
            }],
            function (err) {

                console.log(err);
                err.should.have.property('message').which.equal('error');
                done();
               
            }
        )
    });

    it('passing error', function (done) {

        async([
            function (next) {
                // async result
                setTimeout(function () {
                    next('errorr', 2);
                }, 20);

            }],
            function (err) {

                err.should.have.property('message').which.equal('errorr');
                done();
               
 
            }
        )
    });


    it('timeout error', function (done) {

        async([
            function (next) {
                // async result
                setTimeout(function () {
                    next(null, 2);
                }, 20);

            }],
            function (err) {
                err.should.have.property('message');
                done();
               
 
            },{timeout:10} 
        )
    });

    after(function () {
        console.log('DONE');
    });

});

describe('function-async with promises', function () {

    before(function (cb) {
        cb();
    });


    it('promise', function (done) {

        async([function () {
                return {then: function(fn){ fn(34)}};
            }],
            function (err, result) {

                result.should.be.equal(34);

                done();
            }
            
        )
    });

    after(function () {
        console.log('DONE');
    });

});

describe('function-asyncMap', function () {

    before(function (cb) {
        cb();
    });

    it('parallel flow', function (done) {

        asyncMap(
            [1,2,3]
            ,
            function (item, next) {
                // async result
                setTimeout(function () {
                    next(null, item);
                }, 20);
            }
            ,
            function (err, ...results) {
                console.log(results)
                should.exist(results[2]);

                done(err);
            }
        );
    });

    it('parallel error', function (done) {

        asyncMap([4,2,3],
            function (item, next) {
                // async result
                setTimeout(function () {
                    next('error', item);
                }, 20);
            },
            function (err, ...results) {
                console.log(err, results)

                should.exist(err);

                done();
            }
        );
    });

    after(function () {
        console.log('DONE');
    });

});